WIDTH, HEIGHT = 1024, 768
PPM = 50
GRAVITY = 1000.0
love.window.setMode(WIDTH, HEIGHT)

world = nil

package.path = package.path .. ";../../../spine-runtimes/spine-love/?.lua;../../../spine-runtimes/?.lua;../../lib/?.lua;src/?.lua"


nullImage = love.graphics.newImage(love.image.newImageData(1, 1))

local Scene = require('scene')
local Person = require('person')
local Creature = require('creature')

function love.load()
    love.physics.setMeter(PPM)
    world = love.physics.newWorld(0, 0, false)
    world:setCallbacks(function(fixtureA, fixtureB, contact)
        local objA = fixtureA:getUserData()
        local objB = fixtureB:getUserData()
        if objA.obj.startCollision then
            objA.obj:startCollision(objA, objB, contact, 1)
        end
        if objB.obj.startCollision then
            objB.obj:startCollision(objB, objA, contact, -1)
        end
    end, function(fixtureA, fixtureB, contact)
        local objA = fixtureA:getUserData()
        local objB = fixtureB:getUserData()
        if objA.obj.endCollision then
            objA.obj:endCollision(objA, objB, contact, 1)
        end
        if objB.obj.endCollision then
            objB.obj:endCollision(objB, objA, contact, -1)
        end
    end)
    scene = Scene.load('')
    player = Person.load('player', 400, HEIGHT - 250)
    player:setAnimation('idle')
    citizens = {}
    citizensLimit = 50
    zoom = 1
    love.graphics.setBackgroundColor(118, 221, 255)
    spawnCooldown = 0
    creature = Creature.load(1224, HEIGHT - 50)
    creature:transitionTo('approach', -1)
end

function love.update(dt)
    creature:update(dt)
    if love.keyboard.isDown('d') then
        player:setAnimation('walking', true)
        player.spine.skeleton.flipX = false
        player.spine.skeleton.x = player.spine.skeleton.x + 1
    elseif love.keyboard.isDown('a') then
        player:setAnimation('walking', true)
        player.spine.skeleton.flipX = true
        player.spine.skeleton.x = player.spine.skeleton.x - 1
    else
        player:setAnimation('idle')
    end

    if spawnCooldown >= 0 then
        spawnCooldown = math.max(0, spawnCooldown - dt)
    end

    if #citizens < citizensLimit and spawnCooldown <= 0 then
        local citizen
        if love.math.random() > 0.5 then
            citizen = Person.load('citizen', 0, HEIGHT - 250)
            citizen.dir = 1
            citizen:setAnimation('walking', true)
        else
            citizen = Person.load('citizen', WIDTH, HEIGHT - 250)
            citizen.dir = -1
            citizen.spine.skeleton.flipX = true
            citizen:setAnimation('walking', true)
        end
        citizen.speed = love.math.random(1, 3)
        table.insert(citizens, citizen)
        spawnCooldown = love.math.random(1, 3)
    end

    local newCitizens = {}
    for i, citizen in pairs(citizens) do
        citizen:update(dt)
        if citizen.spine.animation == 'walking' then
            citizen.spine.skeleton.x = citizen.spine.skeleton.x + citizen.dir * citizen.speed
        end
        if not citizen.reapable and citizen.spine.skeleton.x > -100 and citizen.spine.skeleton.x < (WIDTH * 2) + 100 then
            table.insert(newCitizens, citizen)
        end
    end
    citizens = newCitizens

    player:update(dt)
    world:update(dt)
end

function love.draw()
    love.graphics.setColor(255, 255, 255)

    love.graphics.push()

    local speed = 0.125
    local cx,cy = WIDTH/(2*zoom), HEIGHT/(2*zoom)
    love.graphics.scale(zoom)
    love.graphics.translate(cx*speed, cy*speed)
    love.graphics.translate(-player.spine.skeleton.x*speed, (-player.spine.skeleton.y + 134)*speed)
    scene:draw(1)

    love.graphics.pop()
    love.graphics.push()

    local speed = 0.25
    local cx,cy = WIDTH/(2*zoom), HEIGHT/(2*zoom)
    love.graphics.scale(zoom)
    love.graphics.translate(cx*speed, cy*speed)
    love.graphics.translate(-player.spine.skeleton.x*speed, (-player.spine.skeleton.y + 134)*speed)
    scene:draw(2)

    love.graphics.pop()
    love.graphics.push()

    local speed = 0.5
    local cx,cy = WIDTH/(2*zoom), HEIGHT/(2*zoom)
    love.graphics.scale(zoom)
    love.graphics.translate(cx*speed, cy*speed)
    love.graphics.translate(-player.spine.skeleton.x*speed, (-player.spine.skeleton.y + 134)*speed)
    scene:draw(3)

    love.graphics.pop()
    love.graphics.push()

    local speed = 1
    local cx,cy = WIDTH/(2*zoom), HEIGHT/(2*zoom)
    love.graphics.scale(zoom)
    love.graphics.translate(cx*speed, cy*speed)
    love.graphics.translate(-player.spine.skeleton.x*speed, (-player.spine.skeleton.y + 134)*speed)
    scene:draw(4)
    scene:draw(5)

    love.graphics.setLineWidth(5)
    for i, citizen in pairs(citizens) do
        citizen:draw()
    end
    player:draw()
    creature:draw()
    scene:drawObjects()

    love.graphics.pop()
    love.graphics.print(string.format('FPS: %f', love.timer.getFPS()))
end
