local Spine = require('spine-love.spine')
local SpineHelper = require('spiner-helper')

local module = {}
function module.load(name, x, y)
    local spine = SpineHelper.load('pico-spine.json', 0.5)
    local self = setmetatable({
        name = name,
        spine = spine
    }, {
        __index = function(obj, key)
            return module[key]
        end
    })
    self:update(0)

    self.hurtbox = {
        body = love.physics.newBody(world, x, y, 'dynamic'),
        shape = love.physics.newPolygonShape(unpack(self.spine.skeletonBounds:getPolygon(self.spine.skeleton:getAttachment('hurtbox', 'hurtbox'))))
    }
    self.hurtbox.fixture = love.physics.newFixture(self.hurtbox.body, self.hurtbox.shape)
    self.hurtbox.fixture:setSensor(true)
    self.hurtbox.fixture:setUserData({
        name = 'person-hurt',
        obj = self
    })

    spine.skeleton.x = x
    spine.skeleton.y = y

    return self
end

function module:setAnimation(...)
    self.spine:setAnimation(...)
end

function module:update(dt)
    self.spine:update(dt)
    if self.hurtbox then
        self.hurtbox.body:setPosition(self.spine.skeleton.x, self.spine.skeleton.y)
    end
    if self.dead and not self.reapable then
        if not self.deadTimer then
            self.deadTimer = dt
        else
            self.deadTimer = self.deadTimer + dt
        end
        if self.deadTimer > 5 then
            self.reapable = true
        end
    end
end

function module:draw()
    self.spine:draw()
    -- love.graphics.setColor(0, 255, 0)
    -- love.graphics.polygon('line', self.hurtbox.body:getWorldPoints(self.hurtbox.shape:getPoints()))
end

function module:startCollision(me, other, contact, dir)
    if self.name == 'citizen' and me.name == 'person-hurt'
    and self.spine.animation ~= 'knockdown-forward' then
        if other.name == 'creature-aura' then
            self:setAnimation('cower', false)
        elseif other.name == 'creature-attack' then
            self:setAnimation('knockdown-forward')
            self.dead = true
        end
    end
end

function module:endCollision(me, other, contact, dir)
    if self.name == 'citizen' and me.name == 'person-hurt'
    and other.name == 'creature-aura' and self.spine.animation == 'cower' then
        self:setAnimation('walking', true)
        local nx = other.obj.body:getX() - self.spine.skeleton.x
        if nx > 0 then
            self.dir = -1
            self.spine.skeleton.flipX = true
        else
            self.dir = 1
            self.spine.skeleton.flipX = false
        end
    end
end

return module