local Xml = require("xml")
local Json = require("json")

local module = {}
function module.load(filepath)
    local contents, size = love.filesystem.read(filepath .. 'stack.xml')
    local data = Xml.string_to_table(contents)
    local root = data[2]
    local layers = {}
    local self = {
        x = 0,
        y = 0,
        w = root.xarg.w,
        h = root.xarg.h,
        layers = layers
    }
    local numLayers = #root[1]
    for i, layerData in ipairs(root[1]) do
        local layer = {
            name = layerData.xarg.name,
            x = layerData.xarg.x,
            y = layerData.xarg.y,
            src = layerData.xarg.src,
        }
        layer.image = love.graphics.newImage(layer.src)
        layers[numLayers - i + 1] = layer
    end

    contents, size = love.filesystem.read(filepath .. 'objects.json')
    local data = Json:decode(contents)
    local objects = {}
    for i, obj in pairs(data.objects) do
        obj.body = love.physics.newBody(world, obj.x or 0, obj.y or 0)
        obj.shape = love.physics.newPolygonShape(unpack(obj.vertices))
        obj.fixture = love.physics.newFixture(obj.body, obj.shape)
        obj.fixture:setSensor(true)
        obj.fixture:setUserData({name='platform', obj=obj})
        table.insert(objects, obj)
    end
    self.objects = objects
    print(string.format('Loaded %d layers and %d objects in scene', #layers, #objects))

    return setmetatable(self, {
        __index = function(obj, key)
            return module[key]
        end
    })
end

function module:update(dt)
end

function module:draw(index)
    if index then
        local layer = self.layers[index]
        love.graphics.draw(layer.image, self.x, self.y,
            0, 1, 1, layer.x, layer.y)
    else
        for i, layer in pairs(self.layers) do
            love.graphics.draw(layer.image, self.x, self.y,
                0, 1, 1, layer.x, layer.y)
        end
    end
end

function module:drawObjects()
    love.graphics.setColor(0, 255, 0)
    for i, obj in pairs(self.objects) do
        love.graphics.polygon('line', obj.body:getWorldPoints(obj.shape:getPoints()))
    end
end

return module
