local Spine = require("spine-love.spine")

local module = {}

local function nullImageLoader(attachment)
    return nullImage
end

function module.load(dataFile, scale, imageLoader)
    local json = Spine.SkeletonJson.new()
    json.scale = scale
    local skeletonData = json:readSkeletonDataFile(dataFile)
    local self = setmetatable({
        animation = nil
    }, {
        __index = function(obj, key)
            return module[key]
        end
    })
    local skeleton = Spine.Skeleton.new(skeletonData)
    skeleton.x = 0
    skeleton.y = 0
    skeleton.createImage = imageLoader or nullImageLoader
    skeleton.flipX = false
    skeleton.flipY = false
    skeleton.debugBones = false
    skeleton.debugSlots = true
    skeleton:setToSetupPose()
    self.skeleton = skeleton
    self.skeletonData = skeletonData
    self.skeletonTransitions = Spine.AnimationStateData.new(skeletonData)
    self.skeletonState = Spine.AnimationState.new(self.skeletonTransitions)
    self.skeletonBounds = Spine.SkeletonBounds.new()    
    return self
end

function module:setAnimation(name, looping)
    local anim = self.skeletonData:findAnimation(name)
    local currentAnimation = self.skeletonState:getCurrent(0)
    if not currentAnimation or anim ~= currentAnimation.animation then
        if name == 'idle' and not anim then
            self.skeleton:setToSetupPose()
            self.skeletonState:clearTrack(0)
        else
            print('changing animation to', name)
            self.skeletonState:setAnimation(0, anim, looping)
        end
    end
    self.animation = name
end

function module:update(dt)
    self.skeletonState:update(dt)
    self.skeletonState:apply(self.skeleton)
    self.skeleton:updateWorldTransform()
    self.skeletonBounds:update(self.skeleton)
end

function module:draw()
    self.skeleton:draw()
end

return module
